from tornado.websocket import WebSocketHandler


class SocketHandler(WebSocketHandler):
    active_clients = set()

    def open(self):
        SocketHandler.active_clients.add(self)

    def on_close(self):
        SocketHandler.active_clients.remove(self)



import os
import tornado.web
import tornado.ioloop

from .. import config
from .html_handler import HtmlHandler
from .socket_handler import SocketHandler


def broadcast_reload():
    for client in SocketHandler.active_clients:
        client.write_message('reload', binary=False)


def make_app():
    static_config = {'path': config.SERVER_ROOT, 'default_filename': 'index.html'}
    app_config = { 'debug': True, 'serve_traceback': True }

    return tornado.web.Application([
        (r'/(.*\.html)?', HtmlHandler, static_config),
        (r'/ws/socket', SocketHandler),
        (r'/(live-static-server.js)', tornado.web.StaticFileHandler, {'path': os.path.join(os.path.dirname(__file__), 'static')}),
        (r'/(.*)', tornado.web.StaticFileHandler, static_config)
    ], **app_config)


def start_app():
    app = make_app()
    server = app.listen(config.SERVER_PORT)
    print('listening on {}'.format(config.SERVER_PORT))
    tornado.ioloop.IOLoop.current().start()

    return server


def stop_app(app):
    app.stop()



from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler
import tornado.ioloop
import tornado.autoreload

from . import config
from .server.app import broadcast_reload, start_app

app = None
observer = None
loop = None

class LiveServerEventHandler(FileSystemEventHandler):
    def refresh_app():
        global app
        broadcast_reload()

    def on_any_event(self, event):
        global app
        global loop
        print(event)
        loop.add_callback(LiveServerEventHandler.refresh_app)


def watch():
    global observer

    live_server_event_handler = LiveServerEventHandler()
    observer = Observer()
    observer.schedule(live_server_event_handler, config.SERVER_ROOT, recursive=True)
    observer.start()


def main():
    global app
    global loop

    loop = tornado.ioloop.IOLoop.current()
    loop.add_callback(watch)
    app = start_app()
    try:
        loop.start()
    except KeyboardInterrupt:
        print('\nLive Static Server stopped.')
